@@ -1,6 +1,6 @@
 package MooseX::ClassCompositor;
 {
-  $MooseX::ClassCompositor::VERSION = '0.004';
+  $MooseX::ClassCompositor::VERSION = '0.005';
 }
 use Moose;
 # ABSTRACT: a factory that builds classes from roles
@@ -50,8 +50,8 @@ has role_prefixes => (
 );
 
 sub _rewrite_roles {
-  my ($self, @in) = @_;
-  return String::RewritePrefix->rewrite($self->_role_prefixes, @in);
+  my $self = shift;
+  String::RewritePrefix->rewrite($self->_role_prefixes, @_);
 }
 
 
@@ -70,8 +70,7 @@ has serial_counter => (
   init_arg => undef,
 );
 
-has _memoization_table => (
-  is  => 'ro',
+has memoization_table => (
   isa => 'HashRef',
   default  => sub {  {}  },
   traits   => [ 'Hash' ],
@@ -83,6 +82,13 @@ has _memoization_table => (
 );
 
 
+has forbid_meta_role_objects => (
+  is  => 'ro',
+  isa => 'Bool',
+  default => 0,
+);
+
+
 sub class_for {
   my ($self, @args) = @_;
 
@@ -102,7 +108,7 @@ sub class_for {
 
   while (@args) {
     my $name = shift @args;
-    if (ref $name) {
+    if (ref $name eq 'ARRAY') {
       my ($role_name, $moniker, $params) = @$name;
 
       my $full_name = $self->_rewrite_roles($role_name);
@@ -113,6 +119,12 @@ sub class_for {
 
       push @roles, $role_object;
       $name = $moniker;
+    } elsif (blessed $name and $name->DOES('Moose::Meta::Role')) {
+      confess "this class compositor does not allow role objects"
+        if $self->forbid_meta_role_objects;
+
+      push @roles, $name;
+      $name = $name->name;
     } else {
       push @role_class_names, $name;
     }
@@ -125,12 +137,15 @@ sub class_for {
 
   my $name = join q{::}, $self->class_basename, @all_names;
 
-  @role_class_names = (
-    $self->_rewrite_roles(
-      @role_class_names,
-      @{ $self->_fixed_roles },
-    ),
-  );
+  for my $r (@{ $self->_fixed_roles }) {
+    if (blessed $r and $r->DOES('Moose::Meta::Role')) {
+      push @roles, $r;
+    } else {
+      push @role_class_names, $r;
+    }
+  }
+
+  @role_class_names = $self->_rewrite_roles(@role_class_names);
 
   Class::MOP::load_class($_) for @role_class_names;
 
@@ -164,9 +179,11 @@ sub _memoization_key {
   my @k;
   while (@args) {
     my $arg = shift @args;
-    if (ref $arg) {
+    if (ref $arg eq 'ARRAY') {
       my ($role_name, $moniker, $params) = @$arg;
       push @k, "$moniker : { " . __hash_to_string($params) . " }";
+    } elsif (blessed $arg and $arg->DOES('Moose::Meta::Role')) {
+      push @k, $arg->name;
     } else {
       push @k, $arg;
     }
@@ -191,6 +208,7 @@ __PACKAGE__->meta->make_immutable;
 1;
 
 __END__
+
 =pod
 
 =head1 NAME
@@ -199,7 +217,7 @@ MooseX::ClassCompositor - a factory that builds classes from roles
 
 =head1 VERSION
 
-version 0.004
+version 0.005
 
 =head1 SYNOPSIS
 
@@ -268,10 +286,21 @@ expanding role names passed to the compositor's L<class_for> method.
 
 =head2 fixed_roles
 
-This attribute may be initialized with an arrayref of role names.  These roles
-will I<always> be composed in the classes built by the compositor.
+This attribute may be initialized with an arrayref of role names and/or
+L<Moose::Meta::Role> objects.  These roles will I<always> be composed in
+the classes built by the compositor.
 
-These names I<will> be rewritten by the role prefixes.
+Role names (but not Moose::Meta::Role objects) I<will> be rewritten by
+the role prefixes.
+
+=head2 forbid_meta_role_objects
+
+If true, an exception will be raised if a Moose::Meta::Role object is passed to
+C<L</class_for>>.  This is only rarely useful, such as if it's a strict
+requirement that the memoization table of the compositor be serializable and
+its contents reproduceable.
+
+Probably you don't need this.
 
 =head1 METHODS
 
@@ -279,7 +308,8 @@ These names I<will> be rewritten by the role prefixes.
 
   my $class = $compositor->class_for(
 
-    'Role::Name', # <-- will be expanded with role_prefixes
+    'Role::Name',          #  <-- will be expanded with role_prefixes
+    Other::Role->meta,     #  <-- will not be touched
 
     [
       'Param::Role::Name', #  <-- will be expanded with role_prefixes
@@ -289,14 +319,12 @@ These names I<will> be rewritten by the role prefixes.
   );
 
 This method will return a class with the roles passed to it.  They can be given
-either as names (which will be expanded according to C<L</role_prefixes>>) or
-as arrayrefs containing a role name, application name, and hashref of
-parameters.  In the arrayref form, the application name is just a name used to
-uniquely identify this application of a parameterized role, so that they can be
-applied multiple times with each application accounted for internally.
-
-Note that at present, passing Moose::Meta::Role objects is B<not> supported.
-This should change in the future.
+either as names (which will be expanded according to C<L</role_prefixes>>), as
+L<Moose::Meta::Role> objects, or as arrayrefs containing a role name,
+application name, and hashref of parameters.  In the arrayref form, the
+application name is just a name used to uniquely identify this application of
+a parameterized role, so that they can be applied multiple times with each
+application accounted for internally.
 
 =head1 THANKS
 
@@ -324,4 +352,3 @@ This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
 
 =cut
-
